/*
 * File:        arraytests.c
 * Copyright:   (c) 2001 The Regents of the University of California
 * Release:     $Name: release-0-8-0 $
 * Revision:    @(#) $Revision: 1.9 $
 * Date:        $Date: 2002/10/23 17:46:45 $
 * Description: Simple test on the ArrayTest static methods
 *
 */

#include "ArrayTest_ArrayOps.h"
#include "SIDL_header.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

static const char * result_string[] = { "FAIL", "XFAIL", "PASS" };
const int pass = 2;
const int xfail = 1;
const int fail = 0;

static const int TEST_SIZE = 345; /* size of one dimensional arrays */
static const int TEST_DIM1 = 17; /* first dimension of 2-d arrays */
static const int TEST_DIM2 = 13; /* second dimension of 2-d arrays */

static void declare_part( int * part_no ) {
  ++(*part_no);
  printf("PART %d\n", *part_no );
}

static void end_part( int part_no, int result, int *final_result ) {
  printf("RESULT %d %s\n", part_no, result_string[ result ] );
  if ( result < *final_result ) { 
    *final_result = result;
  }
}

static double powTwo(register int32_t i)
{
  register double result = 1;
  if (i >= 0) {
    while (i--) {
      result *= 2;
    }
  }
  else {
    while (i++) {
      result *= 0.5;
    }
  }
  return result;
}

static float fpowTwo(register int32_t i)
{
  register float result = 1;
  static volatile float forcestore;
  if (i >= 0) {
    while (i--) {
      result *= 2.0F;
    }
  }
  else {
    while (i++) {
      result *= 0.5F;
    }
  }
  forcestore = result;
  return forcestore;
}

static struct SIDL_double__array *
makeDTestMatrix(int32_t rows, int32_t cols) {
  int32_t lower[2], upper[2], i, j;
  struct SIDL_double__array *result;
  lower[0] = 0;
  upper[0] = rows - 1;
  lower[1] = 0;
  upper[1] = cols - 1;
  result = SIDL_double__array_createRow(2, lower, upper);
  for(i = 0; i < rows; ++i) {
    lower[0] = i;
    for(j = 0; j < cols; ++j) {
      lower[1] = j;
      switch(rand() % 3) {
      case 0:
        SIDLArrayElem2(result,i,j) = powTwo(i-j);
        break;
      case 1:
        SIDL_double__array_set2(result, i, j, powTwo(i-j));
        break;
      case 2:
        SIDL_double__array_set(result, lower, powTwo(i-j));
        break;
      }
    }
  }
  return result;
}

static struct SIDL_float__array *
makeFTestMatrix(int32_t rows, int32_t cols) {
  int32_t lower[2], upper[2], i, j;
  struct SIDL_float__array *result;
  lower[0] = 0;
  upper[0] = rows - 1;
  lower[1] = 0;
  upper[1] = cols - 1;
  result = SIDL_float__array_createRow(2, lower, upper);
  for(i = 0; i < rows; ++i) {
    lower[0] = i;
    for(j = 0; j < cols; ++j) {
      lower[1] = j;
      switch(rand() % 3) {
      case 0:
        SIDLArrayElem2(result,i,j) = powTwo(i-j);
        break;
      case 1:
        SIDL_float__array_set2(result, i, j, powTwo(i-j));
        break;
      case 2:
        SIDL_float__array_set(result, lower, powTwo(i-j));
        break;
      }
    }
  }
  return result;
}

static struct SIDL_dcomplex__array *
makeDCTestMatrix(int32_t rows, int32_t cols) {
  int32_t lower[2], upper[2], i, j;
  struct SIDL_dcomplex__array *result;
  lower[0] = 0;
  upper[0] = rows - 1;
  lower[1] = 0;
  upper[1] = cols - 1;
  result = SIDL_dcomplex__array_createRow(2, lower, upper);
  for(i = 0; i < rows; ++i) {
    lower[0] = i;
    for(j = 0; j < cols; ++j) {
      struct SIDL_dcomplex tmp;
      tmp.real = powTwo(i);
      tmp.imaginary = powTwo(-j);
      lower[1] = j;
      switch(rand() % 3) {
      case 0:
        SIDLArrayElem2(result,i,j) = tmp;
        break;
      case 1:
        SIDL_dcomplex__array_set2(result, i, j, tmp);
        break;
      case 2:
        SIDL_dcomplex__array_set(result, lower, tmp);
        break;
      }
    }
  }
  return result;
}

static struct SIDL_fcomplex__array *
makeFCTestMatrix(int32_t rows, int32_t cols) {
  int32_t lower[2], upper[2], i, j;
  struct SIDL_fcomplex__array *result;
  lower[0] = 0;
  upper[0] = rows - 1;
  lower[1] = 0;
  upper[1] = cols - 1;
  result = SIDL_fcomplex__array_createRow(2, lower, upper);
  for(i = 0; i < rows; ++i) {
    lower[0] = i;
    for(j = 0; j < cols; ++j) {
      struct SIDL_fcomplex tmp;
      lower[1] = j;
      tmp.real = fpowTwo(i);
      tmp.imaginary = fpowTwo(-j);
      switch(rand() % 3) {
      case 0:
        SIDLArrayElem2(result,i,j) = tmp;
        break;
      case 1:
        SIDL_fcomplex__array_set2(result, i, j, tmp);
        break;
      case 2:
        SIDL_fcomplex__array_set(result, lower, tmp);
        break;
      }
    }
  }
  return result;
}

static int32_t intFunc(const int dimen, const int32_t ind[])
{
  int32_t result = 1;
  int i;
  for(i = 0;i < dimen; ++i){
    result *= (ind[i] + (i + 1));
  }
  return result;
}

static int
next(const int dimen, int32_t ind[], 
     const int32_t lower[], const int32_t upper[])
{
  int i = 0;
  while ((i < dimen) && (++(ind[i]) > upper[i])) {
    ind[i] = lower[i];
    ++i;
  }
  return i < dimen;
}

static struct SIDL_int__array *
makeIntTestMatrix(int dimen)
{
  struct SIDL_int__array *result = NULL;
  static const int32_t lower[] = {0, 0, 0, 0};
  static const int32_t upper[] = {3, 3, 2, 2};
  int32_t ind[] = {0, 0, 0, 0};
  int32_t value;
  result  = SIDL_int__array_createCol(dimen, lower, upper);
  do {
    value = intFunc(dimen, ind);
    switch(rand() % 3) {
    case 0:
      if (dimen == 3) {
        SIDLArrayElem3(result, ind[0], ind[1], ind[2]) = value;
      }
      else {
        SIDLArrayElem4(result, ind[0], ind[1], ind[2], ind[3]) = value;
      }
      break;
    case 1:
      SIDL_int__array_set(result, ind, value);
      break;
    case 2:
      if (dimen == 3) {
        SIDL_int__array_set3(result, ind[0], ind[1], ind[2], value);
      }
      else {
        SIDL_int__array_set4(result, ind[0], ind[1], ind[2], ind[3], value);
      }
      break;
    }
  } while (next(dimen, ind, lower, upper));
  return result;
}

/**
 * Fill the stack with random junk.
 */
int clearstack(int magicNumber) {
  int chunk[2048], i;
  for(i = 0; i < 2048; i++){
    chunk[i] = rand() + magicNumber;
  }
  for(i = 0; i < 16; i++){
    magicNumber += chunk[rand() & 2047];
  }
  return magicNumber;
}

#define MYASSERT( AAA ) \
  declare_part( &part_no ); \
  magicNumber = clearstack(magicNumber); \
  printf("COMMENT: %s\n", #AAA ); \
  if ( AAA ) result = pass; \
  else result = fail;  \
  end_part( part_no, result, &final_result );

int main(int argc, char **argv) {
  int result = pass;
  int final_result = pass;
  int part_no = 0;
  int magicNumber = 1;

  struct SIDL_bool__array     *barray;
  struct SIDL_char__array     *carray;
  struct SIDL_int__array      *iarray;
  struct SIDL_dcomplex__array *dcarray;
  struct SIDL_fcomplex__array *fcarray;
  struct SIDL_long__array     *larray;
  struct SIDL_double__array   *darray;
  struct SIDL_float__array    *farray;
  struct SIDL_string__array   *sarray;

  printf("NPARTS -1\n");

  magicNumber = clearstack(magicNumber);
  barray = ArrayTest_ArrayOps_createBool(TEST_SIZE);
  MYASSERT(barray);
  MYASSERT(ArrayTest_ArrayOps_checkBool(barray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseBool(&barray, TRUE) == TRUE);
  magicNumber = clearstack(magicNumber);
  SIDL_bool__array_deleteRef(barray);
  ArrayTest_ArrayOps_makeBool(218, &barray);
  MYASSERT(ArrayTest_ArrayOps_checkBool(barray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseBool(&barray, FALSE) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_checkBool(barray) == FALSE);
  magicNumber = clearstack(magicNumber);
  SIDL_bool__array_deleteRef(barray);
  ArrayTest_ArrayOps_makeBool(9, &barray);
  MYASSERT(ArrayTest_ArrayOps_reverseBool(&barray, FALSE) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_checkBool(barray) == TRUE);
  magicNumber = clearstack(magicNumber);
  SIDL_bool__array_deleteRef(barray);
  
  magicNumber = clearstack(magicNumber);
  carray = ArrayTest_ArrayOps_createChar(TEST_SIZE);
  MYASSERT(carray);
  MYASSERT(ArrayTest_ArrayOps_checkChar(carray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseChar(&carray, TRUE) == TRUE);
  magicNumber = clearstack(magicNumber);
  SIDL_char__array_deleteRef(carray);
  ArrayTest_ArrayOps_makeChar(218, &carray);
  MYASSERT(ArrayTest_ArrayOps_checkChar(carray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseChar(&carray, FALSE) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_checkChar(carray) == FALSE);
  magicNumber = clearstack(magicNumber);
  SIDL_char__array_deleteRef(carray);
  
  magicNumber = clearstack(magicNumber);
  iarray = ArrayTest_ArrayOps_createInt(TEST_SIZE);
  MYASSERT(iarray);
  MYASSERT(ArrayTest_ArrayOps_checkInt(iarray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseInt(&iarray, TRUE) == TRUE);
  magicNumber = clearstack(magicNumber);
  SIDL_int__array_deleteRef(iarray);
  ArrayTest_ArrayOps_makeInt(218, &iarray);
  MYASSERT(ArrayTest_ArrayOps_checkInt(iarray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseInt(&iarray, FALSE) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_checkInt(iarray) == FALSE);
  magicNumber = clearstack(magicNumber);
  SIDL_int__array_deleteRef(iarray);
  
  magicNumber = clearstack(magicNumber);
  larray = ArrayTest_ArrayOps_createLong(TEST_SIZE);
  MYASSERT(larray);
  MYASSERT(ArrayTest_ArrayOps_checkLong(larray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseLong(&larray, TRUE) == TRUE);
  magicNumber = clearstack(magicNumber);
  SIDL_long__array_deleteRef(larray);
  ArrayTest_ArrayOps_makeLong(218, &larray);
  MYASSERT(ArrayTest_ArrayOps_checkLong(larray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseLong(&larray, FALSE) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_checkLong(larray) == FALSE);
  magicNumber = clearstack(magicNumber);
  SIDL_long__array_deleteRef(larray);
  
  magicNumber = clearstack(magicNumber);
  sarray = ArrayTest_ArrayOps_createString(TEST_SIZE);
  MYASSERT(sarray);
  MYASSERT(ArrayTest_ArrayOps_checkString(sarray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseString(&sarray, TRUE) == TRUE);
  magicNumber = clearstack(magicNumber);
  SIDL_string__array_deleteRef(sarray);
  ArrayTest_ArrayOps_makeString(218, &sarray);
  MYASSERT(ArrayTest_ArrayOps_checkString(sarray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseString(&sarray, FALSE) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_checkString(sarray) == FALSE);
  magicNumber = clearstack(magicNumber);
  SIDL_string__array_deleteRef(sarray);
  
  magicNumber = clearstack(magicNumber);
  darray = ArrayTest_ArrayOps_createDouble(TEST_SIZE);
  MYASSERT(darray);
  MYASSERT(ArrayTest_ArrayOps_checkDouble(darray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseDouble(&darray, TRUE) == TRUE);
  magicNumber = clearstack(magicNumber);
  SIDL_double__array_deleteRef(darray);
  ArrayTest_ArrayOps_makeDouble(218, &darray);
  MYASSERT(ArrayTest_ArrayOps_checkDouble(darray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseDouble(&darray, FALSE) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_checkDouble(darray) == FALSE);
  magicNumber = clearstack(magicNumber);
  SIDL_double__array_deleteRef(darray);
  
  magicNumber = clearstack(magicNumber);
  farray = ArrayTest_ArrayOps_createFloat(TEST_SIZE);
  MYASSERT(farray);
  MYASSERT(ArrayTest_ArrayOps_checkFloat(farray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseFloat(&farray, TRUE) == TRUE);
  magicNumber = clearstack(magicNumber);
  SIDL_float__array_deleteRef(farray);
  ArrayTest_ArrayOps_makeFloat(218, &farray);
  MYASSERT(ArrayTest_ArrayOps_checkFloat(farray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseFloat(&farray, FALSE) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_checkFloat(farray) == FALSE);
  magicNumber = clearstack(magicNumber);
  SIDL_float__array_deleteRef(farray);
  
  magicNumber = clearstack(magicNumber);
  fcarray = ArrayTest_ArrayOps_createFcomplex(TEST_SIZE);
  MYASSERT(fcarray);
  MYASSERT(ArrayTest_ArrayOps_checkFcomplex(fcarray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseFcomplex(&fcarray, TRUE) == TRUE);
  magicNumber = clearstack(magicNumber);
  SIDL_fcomplex__array_deleteRef(fcarray);
  ArrayTest_ArrayOps_makeFcomplex(218, &fcarray);
  MYASSERT(ArrayTest_ArrayOps_checkFcomplex(fcarray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseFcomplex(&fcarray, FALSE) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_checkFcomplex(fcarray) == FALSE);
  magicNumber = clearstack(magicNumber);
  SIDL_fcomplex__array_deleteRef(fcarray);
  
  magicNumber = clearstack(magicNumber);
  dcarray = ArrayTest_ArrayOps_createDcomplex(TEST_SIZE);
  MYASSERT(dcarray);
  MYASSERT(ArrayTest_ArrayOps_checkDcomplex(dcarray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseDcomplex(&dcarray, TRUE) == TRUE);
  magicNumber = clearstack(magicNumber);
  SIDL_dcomplex__array_deleteRef(dcarray);
  ArrayTest_ArrayOps_makeDcomplex(218, &dcarray);
  MYASSERT(ArrayTest_ArrayOps_checkDcomplex(dcarray) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_reverseDcomplex(&dcarray, FALSE) == TRUE);
  MYASSERT(ArrayTest_ArrayOps_checkDcomplex(dcarray) == FALSE);
  magicNumber = clearstack(magicNumber);
  SIDL_dcomplex__array_deleteRef(dcarray);

  magicNumber = clearstack(magicNumber);
  darray = makeDTestMatrix(TEST_DIM1,TEST_DIM2);
  MYASSERT(ArrayTest_ArrayOps_check2Double(darray) == TRUE);
  SIDL_double__array_deleteRef(darray);

  magicNumber = clearstack(magicNumber);
  farray = makeFTestMatrix(TEST_DIM1,TEST_DIM2);
  MYASSERT(ArrayTest_ArrayOps_check2Float(farray) == TRUE);
  SIDL_float__array_deleteRef(farray);

  magicNumber = clearstack(magicNumber);
  dcarray = makeDCTestMatrix(TEST_DIM1,TEST_DIM2);
  MYASSERT(ArrayTest_ArrayOps_check2Dcomplex(dcarray) == TRUE);
  SIDL_dcomplex__array_deleteRef(dcarray);

  magicNumber = clearstack(magicNumber);
  fcarray = makeFCTestMatrix(TEST_DIM1,TEST_DIM2);
  MYASSERT(ArrayTest_ArrayOps_check2Fcomplex(fcarray) == TRUE);
  SIDL_fcomplex__array_deleteRef(fcarray);

  magicNumber = clearstack(magicNumber);
  iarray = makeIntTestMatrix(3);
  MYASSERT(ArrayTest_ArrayOps_check3Int(iarray) == TRUE);
  SIDL_int__array_deleteRef(iarray);

  magicNumber = clearstack(magicNumber);
  iarray = makeIntTestMatrix(4);
  MYASSERT(ArrayTest_ArrayOps_check4Int(iarray) == TRUE);
  SIDL_int__array_deleteRef(iarray);

  printf("TEST_RESULT %s\n", result_string[ final_result ] );
  return 0;
}
