!
! File:       exceptionclient.F90
! Copyright:  (c) 2001-2002 The Regents of the University of California
! Release:    $Name: release-0-8-0 $
! Revision:   @(#) $Revision: 1.3 $
! Date:       $Date: 2002/12/04 19:46:41 $
! Description:Simple F90 exception test client
!
!

subroutine starttest(number)
  implicit none
  integer (selected_int_kind(9)) :: number
  write (6, 100) number
100 format ('PART ', I4)
end subroutine starttest

subroutine reporttest(test, number, pass, fail)
  implicit none
  integer (selected_int_kind(9)) :: number, pass, fail
  logical                        :: test
  if (test) then
     write (6, 100) number, 'PASS'
     pass = pass + 1
  else
     write (6, 100) number, 'FAIL'
     fail = fail + 1
  endif
100 format ('RESULT', 1x, i2, 1x, A4)
  number = number + 1
end subroutine reporttest

subroutine reportexc(exc)
  implicit none
  integer (selected_int_kind(18)) :: exc
  character (len=100)             :: msg
  character (len=1024)            :: trace

  call SIDL_BaseException_getNote_m(exc, msg)
  write (6, 100) msg
  call SIDL_BaseException_getTrace_m(exc, trace)
  write (6, 110) trace
100 format (1x, a100)
110 format (1x, a1024)
  ! ToDo...Shouldn't the msg string be free'd (e.g., SIDL_String_Free)?
  ! ToDo...Shouldn't the trace string be free'd (e.g., SIDL_String_Free)?
end subroutine reportexc

subroutine testnone(fib, test, pass, fail)
  implicit none
  integer (selected_int_kind(18)) :: fib
  integer (selected_int_kind(9))  :: test, pass, fail
  integer (selected_int_kind(18)) :: retval
  integer (selected_int_kind(18)) :: exc

  call starttest(test)
  call ExceptionTest_Fib_getFib_m(fib, 10, 25, 200, 0, retval, exc)
  if (exc .eq. 0) then
     call reporttest(.true., test, pass, fail)
     write (6, 100) retval
  else
     call reporttest(.false., test, pass, fail)
     call reportexc(exc)
     call ExceptionTest_Fib_deleteRef_m(exc)
  endif
100 format ('fib= ', I4)
end subroutine testnone

subroutine testneg(fib, test, pass, fail)
  implicit none
  integer (selected_int_kind(18)) :: fib
  integer (selected_int_kind(9))  :: test, pass, fail
  integer (selected_int_kind(18)) :: retval
  integer (selected_int_kind(18)) :: exc
  logical                         :: isone

  call starttest(test)
  call ExceptionTest_Fib_getFib_m(fib, -1, 10, 10, 0, retval, exc)
  if (exc .eq. 0) then
     call reporttest(.false., test, pass, fail)
     write (6, 100) retval
  else
     call SIDL_BaseException_isType_m(exc, &
                               'ExceptionTest.NegativeValueException', isone)
     if (isone .eqv. .true.) then
        call reporttest(.true., test, pass, fail)
     else
        call reporttest(.false., test, pass, fail)
     endif
     call reportexc(exc)
     call ExceptionTest_Fib_deleteRef_m(exc)
  endif
100 format ('fib= ', I4)
end subroutine testneg

subroutine testdeep(fib, test, pass, fail)
  implicit none
  integer (selected_int_kind(18)) :: fib
  integer (selected_int_kind(9))  :: test, pass, fail
  integer (selected_int_kind(18)) :: retval
  integer (selected_int_kind(18)) :: exc
  logical                         :: isone

  call starttest(test)
  call ExceptionTest_Fib_getFib_m (fib, 10, 1, 100, 0, retval, exc)
  if (exc .eq. 0) then
     call reporttest(.false., test, pass, fail)
     write (6, 100) retval
  else
     call SIDL_BaseException_isType_m(exc, 'ExceptionTest.TooDeepException', &
                                       isone)
     if (isone .eqv. .true.) then
        call reporttest(.true., test, pass, fail)
     else
        call reporttest(.false., test, pass, fail)
     endif
     call reportexc(exc)
     call ExceptionTest_Fib_deleteRef_m(exc)
  endif
100 format ('fib= ', I4)
end subroutine testdeep

subroutine testbig(fib, test, pass, fail)
  implicit none
  integer (selected_int_kind(18)) :: fib
  integer (selected_int_kind(9))  :: test, pass, fail
  integer (selected_int_kind(18)) :: retval
  integer (selected_int_kind(18)) :: exc
  logical                         :: isone

  call starttest(test)
  call ExceptionTest_Fib_getFib_m(fib, 10, 100, 1, 0, retval, exc)
  if (exc .eq. 0) then
     call reporttest(.false., test, pass, fail)
     write (6, 100) retval
  else
     call SIDL_BaseException_isType_m(exc, 'ExceptionTest.TooBigException', &
                                       isone)
     if (isone .eqv. .true.) then
        call reporttest(.true., test, pass, fail)
     else
        call reporttest(.false., test, pass, fail)
     endif
     call reportexc(exc)
     call ExceptionTest_Fib_deleteRef_m(exc)
  endif
100 format ('fib= ', I4)
end subroutine testbig


program exceptionclient
  implicit none
  integer (selected_int_kind(9))  :: test, pass, fail
  integer (selected_int_kind(18)) :: fib
  integer (selected_int_kind(18)) :: retval

  call ExceptionTest_Fib__create_m(fib)

  test = 1
  pass = 0
  fail = 0

  write(6,130) 4
  write(6,110)
  write(6,120) 'No Exception test            '
  call testnone(fib, test, pass, fail)
  write(6,110)
  write(6,120) 'Negative Value Exception test'
  call testneg(fib, test, pass, fail)
  write(6,110)
  write(6,120) 'Too Deep Exception test      '
  call testdeep(fib, test, pass, fail)
  write(6,110)
  write(6,120) 'Too Big Exception test       '
  call testbig(fib, test, pass, fail)

  call ExceptionTest_Fib_deleteRef_m(fib)

  write(6, 110) 
  if ((fail .eq. 0) .and. (pass .eq. 4)) then
     write(6, 100) 'PASS'
  else
     write(6, 100) 'FAIL'
  endif
100 format ('TEST_RESULT', 1x, a4)
110 format (' ')
120 format ('COMMENT:', 1x, a30)
130 format ('NPARTS', 1x, i4)
end program exceptionclient
