!
! File:       sorttest.f90
! Copyright:  (c) 2002 The Regents of the University of California
! Release:    $Name: release-0-8-0 $
! Revision:   @(#) $Revision: 1.4 $
! Date:       $Date: 2002/12/04 19:46:44 $
! Description:Exercise the FORTRAN interface for sorting
!
!
#include "sort_SortingAlgorithm_fAbbrev.h"
#include "sort_MergeSort_fAbbrev.h"
#include "sort_QuickSort_fAbbrev.h"
#include "sort_HeapSort_fAbbrev.h"
#include "sort_SortTest_fAbbrev.h"

subroutine starttest(number)
  implicit none
  integer (selected_int_kind(9)) :: number
  write (6, 100) number
100 format ('PART ', I4)
end subroutine starttest

subroutine reporttest(test, number, pass, fail)
  implicit none
  integer (selected_int_kind(9)) :: number, pass, fail
  logical                        :: test
  if (test) then
     write (6, 100) number, 'PASS'
     pass = pass + 1
  else
     write (6, 100) number, 'FAIL'
     fail = fail + 1
  endif
100 format ('RESULT', 1x, i2, 1x, A4)
  number = number + 1
end subroutine reporttest

subroutine testsort(test, pass, fail)
  implicit none
  integer (selected_int_kind(18)) :: algs, merge, quick, heap, alg
  integer (selected_int_kind(9))  :: test, pass, fail
  logical                         :: retval

  call sort_SortingAlgorithm__array_create1d_m(3, algs)
  call starttest(test)
  call sort_MergeSort__create_m(merge)
  call reporttest(merge .ne. 0, test, pass, fail)
  call starttest(test)
  call sort_QuickSort__create_m(quick)
  call reporttest(quick .ne. 0, test, pass, fail)
  call starttest(test)
  call sort_HeapSort__create_m(heap)
  call reporttest(heap .ne. 0, test, pass, fail)

  call sort_SortingAlgorithm__cast_m(merge, 'sort.SortingAlgorithm', alg)
  call sort_SortingAlgorithm__array_set1_m(algs, 0, alg)
  call sort_SortingAlgorithm__cast_m(heap, 'sort.SortingAlgorithm', alg)
  call sort_SortingAlgorithm__array_set1_m(algs, 1, alg)
  call sort_SortingAlgorithm__cast_m(quick, 'sort.SortingAlgorithm', alg)
  call sort_SortingAlgorithm__array_set1_m(algs, 2, alg)
  ! remove extraneous references
  call sort_MergeSort_deleteRef_m(merge)
  call sort_QuickSort_deleteRef_m(quick)
  call sort_HeapSort_deleteRef_m(heap)

  call starttest(test)
  call sort_SortTest_stressTest_m(algs, retval)
  call reporttest(retval, test, pass, fail)

end subroutine testsort

program sorttest
  implicit none
  integer (selected_int_kind(9)) :: test, pass, fail
  test = 1
  pass = 0
  fail = 0
  write(6,120) 4
  write(6,110) 'Sort tests'
  call testsort(test, pass, fail)
  if ((fail .eq. 0) .and. (pass .eq. 4)) then
     write(6, 100) 'PASS'
  else
     write(6, 100) 'FAIL'
  endif
100 format ('TEST_RESULT', 1x, a4)
110 format ('COMMENT:', 1x, a20)
120 format ('NPARTS', 1x, i4)
end program sorttest
